/*
 *  This file contains the RBTX4925 console IO package.
 */

#include <bsp.h>
#include <rtems/bspIo.h>
#include <rtems/libio.h>
#include <ctype.h>
#include <rtems/termiostypes.h>
#include <libcpu/loongson3a.h>

//BSP_CONSOLE_PORT_VGABSP_CONSOLE_PORT_COM1BSP_CONSOLE_PORT_COM2
int BSPConsolePort = BSP_CONSOLE_PORT_VGA;
int BSPPrintkPort=BSP_CONSOLE_PORT_COM1;
int BSPUartBaud=115200;


extern int BSP_kbd_polled_read( int minor );
extern void video_putc( const char c );
extern int fb_init( void );
extern void BSP_uart_init( int uart, unsigned long baud, unsigned long databits, unsigned long parity, unsigned long stopbits, int hwFlow );
extern void BSP_uart_polled_write( int uart, int val );
extern int  BSP_uart_polled_read( int uart );


static int loongson3a_first_open( int major, int minor, void *arg )
{
	return 0;
}

static int loongson3a_last_close( int major, int minor, void *arg )
{
	return 0;
}

static int loongson3a_poll_read( int minor )
{
	if(BSPConsolePort==BSP_CONSOLE_PORT_VGA)
	{
		return BSP_kbd_polled_read(BSPConsolePort);
	}
	else
	{
		return BSP_uart_polled_read(BSPConsolePort);
	}
}

static int loongson3a_poll_write( int minor, const char *buf, int n )
{
	int i=0;
	
	if(BSPConsolePort==BSP_CONSOLE_PORT_VGA)
	{
		for ( i = 0; i < n; i++ )
		{
			video_putc( buf[i] );
		}
	}
	else
	{
		for ( i = 0; i < n; i++ )
		{
			BSP_uart_polled_write( BSPConsolePort,buf[i] );
		}
	}
	return 0;
}

static int loongson3a_set_attributes( int minor, const struct termios *t )
{
	return 0;
}


/*  console_initialize
 *
 *  This routine initializes the console IO driver.
 */
rtems_device_driver console_initialize(
  rtems_device_major_number  major,
  rtems_device_minor_number  minor,
  void                      *arg
)
{
  rtems_status_code status;
  rtems_termios_initialize();

  status = rtems_io_register_name(
    "/dev/console",
    major,
    (rtems_device_minor_number) 0
  );

  if (status != RTEMS_SUCCESSFUL)
    rtems_fatal_error_occurred(status);

  if(BSPConsolePort == BSP_CONSOLE_PORT_VGA)
  {
  	fb_init();
  }
  else
  {
	BSP_uart_init( BSPConsolePort, BSPUartBaud, 8, 0, 0, 0 );
  }
  
  return RTEMS_SUCCESSFUL;
}


/*
 *  Open entry point
 */
rtems_device_driver console_open(
  rtems_device_major_number major,
  rtems_device_minor_number minor,
  void                    * arg
)
{
	static rtems_termios_callbacks callbacks =
	{
		loongson3a_first_open,						  /* firstOpen */
		loongson3a_last_close,						  /* lastClose */
		loongson3a_poll_read,				  /* pollRead */
		loongson3a_poll_write,			  /* write */
		loongson3a_set_attributes,		/* setAttributes */
		NULL,						  /* stopRemoteTx */
		NULL,						  /* startRemoteTx */
		TERMIOS_POLLED							  /* outputUsesInterrupts */
	 
	};
	return rtems_termios_open( major, minor, arg, &callbacks);
}

/*
 *  Close entry point
 */
rtems_device_driver console_close(
  rtems_device_major_number major,
  rtems_device_minor_number minor,
  void                    * arg
)
{
  return rtems_termios_close( arg );
}

/*
 * read bytes from the serial port. We only have stdin.
 */
rtems_device_driver console_read(
  rtems_device_major_number major,
  rtems_device_minor_number minor,
  void                    * arg
)
{
  return rtems_termios_read( arg );
}

/*
 * write bytes to the serial port. Stdout and stderr are the same.
 */
rtems_device_driver console_write(
  rtems_device_major_number major,
  rtems_device_minor_number minor,
  void                    * arg
)
{
	return rtems_termios_write ( arg );
}

/*
 *  IO Control entry point
 */
rtems_device_driver console_control(
  rtems_device_major_number major,
  rtems_device_minor_number minor,
  void                    * arg
)
{
  return rtems_termios_ioctl( arg );
}

void Loongson3a_output_char( char c )
{
	//outbyte( c );
	if(BSPPrintkPort==BSP_CONSOLE_PORT_VGA)
	{
		video_putc(c);
	}
	else
	{
		BSP_uart_polled_write(BSPPrintkPort,c);
	}
}
char Loongson3a_input_char( void )
{
	//return inbyte();
	int tmp;
	if(BSPPrintkPort==BSP_CONSOLE_PORT_VGA)
	{
		tmp=BSP_kbd_polled_read(BSPPrintkPort);
	}
	else
	{
		tmp=BSP_uart_polled_read(BSPPrintkPort);
	}
	tmp=tmp&0xff;
	return (char)tmp;
}
BSP_output_char_function_type            	BSP_output_char = Loongson3a_output_char;
BSP_polling_getchar_function_type      	BSP_poll_char = Loongson3a_input_char;

